@@ -4,6 +4,13 @@
 #'   You may use the key word \code{time} in your equations for non-autonomous
 #'   ODEs.
 #' @param forcings Character vector with the names of the forcings
+#' @param events data.frame of events with columns "var" (character, the name of the state to be
+#' affected), "time" (numeric or character, time point), 
+#' "value" (numeric or character, value), "method" (character, either
+#' "replace" or "add"). See \link[deSolve]{events}. If "var" and "time" are
+#' characters, their values need to be speciefied in the parameter vector
+#' when calling \code{\link{odeC}}. An event function is generated and compiled
+#' with the ODE.
 #' @param fixed  character vector with the names of parameters (initial values
 #'   and dynamic) for which no sensitivities are required (will speed up the
 #'   integration).
@@ -60,7 +67,7 @@
 #' odeC(yini, times, func, parms)
 #' }
 #' @export
-funC <- function(f, forcings = NULL, fixed = NULL, outputs=NULL, 
+funC <- function(f, forcings = NULL, events = NULL, fixed = NULL, outputs=NULL, 
                  jacobian=c("none", "full", "inz.lsodes", "jacvec.lsodes"), 
                  rootfunc = NULL, boundary = NULL, 
                  compile = TRUE, fcontrol = c("nospline", "einspline"),
@@ -93,14 +100,15 @@ funC <- function(f, forcings = NULL, fixed = NULL, outputs=NULL,
     forc.t.replace <- paste0("xdot[", 1:length(forcings.t)-1, "]")
   }
   
+     
   ## Analyze f by parser
-  
   variables <- names(f)
-  symbols <- getSymbols(c(f, rootfunc, constraints))
+  symbols <- getSymbols(c(f, rootfunc, constraints, as.character(events[["value"]]), as.character(events[["time"]])))
   parameters <- symbols[!symbols%in%c(variables, forcings, names(constraints), names(rootfunc), "time")]
+
+
   jac <- NULL
   inz <- NULL
-  
   jacobian <- match.arg(jacobian)
   if(jacobian != "none") jac  <- jacobianSymb(f)
   if(jacobian %in% c("inz.lsodes", "jacvec.lsodes")) {
@@ -125,6 +133,7 @@ funC <- function(f, forcings = NULL, fixed = NULL, outputs=NULL,
   ## ------------ deSolve: write code -------------
   
   deSolveSyntax <- function(f) {
+    f <- replaceNumbers(f)
     f <- replaceOperation("^", "pow", f)
     f <- replaceOperation("**", "pow", f)
     f <- replaceSymbols(variables, paste0("y[", 1:length(variables) - 1, "]"), f)
@@ -148,6 +157,26 @@ funC <- function(f, forcings = NULL, fixed = NULL, outputs=NULL,
   if(!is.null(rootfunc))
     rootfunc <- deSolveSyntax(rootfunc)
   
+  if(!is.null(events)) {
+    var <- deSolveSyntax(as.character(events[["var"]]))
+    time <- deSolveSyntax(as.character(events[["time"]]))
+    value <- deSolveSyntax(as.character(events[["value"]]))
+    method <- as.character(events[["method"]])
+    time.unique <- unique(time)
+    eventsfn <- sapply(1:length(time.unique), function(i) {
+      t <- time.unique[i]
+      paste0("\t if(*t == ", t, " & eventcounter[", i-1, "] == 0) {\n",
+             paste(ifelse(method[time == t] == "replace", 
+                    paste0("\t\t", var[time == t], " = ", value[time == t], ";"),
+                    paste0("\t\t", var[time == t], " = ", var[time == t], " + ", value[time == t], ";")
+                    ), collapse = "\n"),
+             "\n",
+             "\t\t", "eventcounter[", i-1, "] = 1.;\n",
+             "\t }\n"
+      )
+    })
+  }
+  
   
   mypath <- system.file(package="cOde")
   splinefile <- paste0("cat ", mypath,"/code/splineCreateEvaluate.c")
@@ -162,6 +191,13 @@ funC <- function(f, forcings = NULL, fixed = NULL, outputs=NULL,
     definitions <- c(definitions, paste0("#define ", forcings, " forc[", 0:(di-1),"]"))
   
   
+  
+  # First unload possible loaded DLL before writing C file (otherwise we have problems on Windows)
+  .so <- .Platform$dynlib.ext
+  test <- try(dyn.unload(paste0(dllname, .so)), silent = TRUE)
+  if (!inherits(test, "try-error")) message(paste("A shared object with name", dllname, "already existed and was overwritten."))
+  
+  # Write to C file
   filename <- paste0(dllname, ".c")
   sink(filename)
   cat("/** Code auto-generated by cOde", as.character(packageVersion("cOde")), "**/\n")
@@ -170,6 +206,7 @@ funC <- function(f, forcings = NULL, fixed = NULL, outputs=NULL,
   cat(paste("static double parms[", dv+dp,"];\n", sep=""))
   cat(paste("static double forc[", di,"];\n", sep=""))
   cat(paste("static double cons[", dc,"];\n", sep=""))
+  if (!is.null(events)) cat(paste("static double eventcounter[", length(eventsfn), "];\n", sep = ""))
   cat("static double range[2];\n")
   cat("\n")
   cat(paste("#define nGridpoints",nGridpoints,"\n"))
@@ -190,6 +227,9 @@ funC <- function(f, forcings = NULL, fixed = NULL, outputs=NULL,
   cat(paste0("void ", modelname, "_initmod(void (* odeparms)(int *, double *)) {\n"))
   cat(paste("\t int N=", dv+dp,";\n",sep=""))
   cat("\t odeparms(&N, parms);\n")
+  if (!is.null(events)) {
+    cat("\t for(int i=0; i<", length(eventsfn), "; ++i) eventcounter[i] = 0;\n", sep = "")
+  }
   cat("}\n")
   cat("\n")
   cat(paste0("void ", modelname, "_initforc(void (* odeforcs)(int *, double *)) {\n"))
@@ -289,6 +329,18 @@ funC <- function(f, forcings = NULL, fixed = NULL, outputs=NULL,
     
   }
   
+  if (!is.null(events)) {
+    
+    cat("/** Event function **/\n")
+    cat(paste0("void ", modelname, "_myevent(int *n, double *t, double *y) {\n"))
+    cat("\n")
+    cat(eventsfn, sep = "\n")
+    cat("\n")
+    cat("}\n")
+    
+    
+  }
+  
   if(!is.null(rootfunc)) {
     
     cat("/** Root function **/\n")
@@ -376,6 +428,11 @@ funC <- function(f, forcings = NULL, fixed = NULL, outputs=NULL,
                    sundialsJac(f, variables, parameters, dllname))
     }
     
+    # First unload possible loaded DLL before writing C file (otherwise we have problems on Windows)
+    .so <- .Platform$dynlib.ext
+    test <- try(dyn.unload(paste0(dllname, .so)), silent = TRUE)
+    if (!inherits(test, "try-error")) message(paste("A shared object with name", dllname, "already existed and was overwritten."))
+    
     
     ## Write source code
     filename <- paste0(dllname, ".cpp")
@@ -401,6 +458,7 @@ funC <- function(f, forcings = NULL, fixed = NULL, outputs=NULL,
   attr(f, "variables") <- variables
   attr(f, "parameters") <- parameters
   attr(f, "forcings") <- forcings
+  attr(f, "events") <- events
   attr(f, "outputs") <- outputs
   attr(f, "jacobian") <- jacobian
   attr(f, "inz") <- inz
@@ -447,11 +505,11 @@ compileAndLoad <- function(filename, dllname, fcontrol, verbose) {
   }
   
   .so <- .Platform$dynlib.ext
-  dllname <- paste0(dllname, .so)
-  soExists <- file.exists(dllname)
+  mydllname <- paste0(dllname, .so)
+  soExists <- file.exists(mydllname)
   if (soExists) {
-    try(dyn.unload(dllname), silent = TRUE)
-    dyn.load(dllname)
+    try(dyn.unload(mydllname), silent = TRUE)
+    dyn.load(mydllname)
   }
 }
 
@@ -836,18 +894,35 @@ odeC <- function(y, times, func, parms, ...) {
     return(out)
   }
   
-  # deSolve
+  ## deSolve ----------------------------------------------------------------------
   nGridpoints <- attr(func, "nGridpoints")
   modelname <- attr(func, "modelname")
+  
+  # Evaluate initial values and parameters
+  y <- y[attr(func, "variables")]
+  parms <- parms[attr(func, "parameters")]
+  parms <- c(parms, rep(0, length(y)))
+  
+  # First handle events to get additional time points
+  events <- attr(func, "events")
+  eventlist <- NULL
+  if (!is.null(events)) {
+    
+    eventfunc <- paste(func, "myevent", sep = "_")
+    time.expr <- parse(text = paste0("c(", paste(as.character(events[["time"]]), collapse = ", "), ")"))
+    # evaluate event parameters and times
+    eventtime <- with(as.list(parms), eval(time.expr))
+    eventlist <- list(func = eventfunc, time = sort(unique(eventtime)))
+    
+  }
+  
+  
+  #times <- sort(union(times, eventlist[["time"]]))
   times.inner <- seq(min(c(times, 0)), max(times), len=nGridpoints)
   times.inner <- sort(unique(c(times, times.inner)))
   which.times <- match(times, times.inner)
   yout <- c(attr(func, "forcings"), names(attr(func, "outputs")))
   
-  y <- y[attr(func, "variables")]
-  parms <- parms[attr(func, "parameters")]
-  parms <- c(parms, rep(0, length(y)))
-  
   arglist <- list(y = y, times = times.inner, func = paste0(func, "_derivs"), parms = parms, dllname = modelname, initfunc = paste0(func, "_initmod"))
   
   
@@ -874,11 +949,15 @@ odeC <- function(y, times, func, parms, ...) {
   if (!is.null(yout)) {
     arglist <- c(arglist, list(nout = length(yout), outnames = yout))
   }
+   
   
   # Replace arguments and add new ones
   moreargs <- list(...)
-  if(any(names(moreargs)=="forcings") & attr(func, "fcontrol") == "einspline") 
-    moreargs <- moreargs[-which(names(moreargs) == "forcings")]
+  if(length(moreargs) > 0) {
+    if (any(names(moreargs)=="forcings") & attr(func, "fcontrol") == "einspline") {
+      moreargs <- moreargs[-which(names(moreargs) == "forcings")]
+    }
+  }
   
   i <- match(names(moreargs), names(arglist))
   is.overlap <- which(!is.na(i))
@@ -886,11 +965,16 @@ odeC <- function(y, times, func, parms, ...) {
   arglist[i[is.overlap]] <- moreargs[is.overlap]
   arglist <- c(arglist, moreargs[is.new])
   
+  # Merge events and event time points
+  arglist[["events"]] <- c(arglist[["events"]], eventlist)
+   
+  
   #loadDLL(func)
   
   out <- do.call(deSolve::ode, arglist)
-  out.index <- unique(c(which.times[which.times <= nrow(out)], nrow(out)))
-  out <- matrix(out[out.index, ], nrow = length(out.index), dimnames = list(NULL, colnames(out)))
+  out <- out[out[, 1] >= min(times.inner) & out[, 1] <= max(times.inner), ]
+  #out.index <- unique(c(which.times[which.times <= nrow(out)], nrow(out)))
+  #out <- matrix(out[out.index, ], nrow = length(out.index), dimnames = list(NULL, colnames(out)))
   
   return(out)
   
